import React, { useState, useEffect, useCallback, useContext } from "react";
import {
  SimpleAccordion,
  ConfirmationDialog,
  useAPIMethod
} from "@ess-ics/ce-ui-common";
import {
  Button,
  CircularProgress,
  TextField,
  Tooltip,
  Typography,
  Autocomplete,
  Alert
} from "@mui/material";
import { useUpdateIoc } from "../../../api/SwaggerApi";
import { useTypingTimer } from "../../common/SearchBoxFilter/TypingTimer";
import AccessControl from "../../auth/AccessControl";
import { apiContext } from "../../../api/DeployApi";

export default function IOCDetailAdmin({
  ioc,
  getIOC,
  resetTab,
  buttonDisabled
}) {
  const [gitId, setGitId] = useState(ioc.gitProjectId);

  const [nameQuery, onNameKeyUp] = useTypingTimer({ interval: 500 });
  const [name, setName] = useState({
    uuid: ioc.namingUuid,
    description: ioc.description,
    name: ioc.namingName
  });

  // for the dialog
  const [open, setOpen] = useState(false);
  const [error, setError] = useState();

  function onError(message) {
    setError(message);
  }

  const client = useContext(apiContext);

  const {
    value: allowedGitProjects,
    wrapper: getAllowedGitProjects,
    loading: loadingAllowedGitProjects,
    dataReady: dataReadyAllowedGitProjects
  } = useAPIMethod({
    fcn: client.apis.Git.listProjects,
    call: false
  });

  const {
    value: names,
    wrapper: getNames,
    loading: loadingNames,
    dataReady
  } = useAPIMethod({
    fcn: client.apis.Naming.fetchIOCByName,
    call: false
  });

  const requiredDataMissing = useCallback(() => !gitId || !name, [gitId, name]);

  const noModification = useCallback(
    () =>
      gitId === ioc.gitProjectId &&
      name?.uuid === ioc.namingUuid &&
      name?.description === ioc.description &&
      name?.name === ioc.namingName,
    [gitId, name, ioc]
  );

  const [uioc, actionUpdateIoc] = useUpdateIoc(ioc.id, onError);

  useEffect(() => {
    getAllowedGitProjects();
  }, [getAllowedGitProjects]);

  useEffect(() => {
    getNames(nameQuery);
  }, [nameQuery, getNames]);

  // when user clicks Submit button a dialog should open
  const onSubmit = (event) => {
    event.preventDefault();
    setOpen(true);
  };

  const onClose = useCallback(() => {
    setOpen(false);
  }, [setOpen]);

  const onConfirm = useCallback(() => {
    actionUpdateIoc({
      gitProjectId: gitId,
      namingUuid: name ? name.uuid : null
    });
  }, [actionUpdateIoc, gitId, name]);

  useEffect(() => {
    if (uioc) {
      getIOC();
      resetTab();
    }
  }, [uioc, getIOC, resetTab]);

  const iocIsDeployed = ioc.activeDeployment;

  let nameDisabledTitle = "";
  if (iocIsDeployed) {
    nameDisabledTitle =
      "IOC cannot be renamed while deployed - please undeploy first";
  }

  function nameAutocomplete(disabled) {
    const isDisabled = disabled || iocIsDeployed;
    const loading = (loadingNames || !dataReady) && !isDisabled;
    return (
      <Tooltip title={nameDisabledTitle}>
        <Autocomplete
          autoHighlight
          id="namingName"
          options={names ?? []}
          loading={loading}
          clearOnBlur={false}
          defaultValue={name}
          getOptionLabel={(option) => {
            return option?.name;
          }}
          renderInput={(params) => (
            <TextField
              {...params}
              label="IOC name"
              variant="outlined"
              required
              InputProps={{
                ...params.InputProps,
                endAdornment: (
                  <React.Fragment>
                    {loading ? (
                      <CircularProgress
                        color="inherit"
                        size={20}
                      />
                    ) : null}
                    {params.InputProps.endAdornment}
                  </React.Fragment>
                )
              }}
            />
          )}
          onChange={(event, value, reason) => {
            setName(value);
            setError(null);
          }}
          onInputChange={(event, value, reason) => {
            event && onNameKeyUp(event.nativeEvent);
          }}
          disabled={isDisabled}
          autoSelect
        />
      </Tooltip>
    );
  }

  function gitRepoAutocomplete(disabled) {
    const isDisabled = disabled || iocIsDeployed;
    const loading =
      (loadingAllowedGitProjects || !dataReadyAllowedGitProjects) &&
      !isDisabled;
    return (
      <Autocomplete
        autoHighlight
        id="gitId"
        options={allowedGitProjects ?? []}
        loading={loading}
        clearOnBlur={false}
        defaultValue={{
          id: gitId,
          url: ioc.sourceUrl
        }}
        getOptionLabel={(option) => {
          return option.url;
        }}
        onChange={(event, value, reason) => {
          setGitId(value?.id);
          setError(null);
        }}
        renderInput={(params) => (
          <TextField
            {...params}
            label="Git repository"
            variant="outlined"
            fullWidth
            required
            InputProps={{
              ...params.InputProps,
              endAdornment: (
                <React.Fragment>
                  {loading ? (
                    <CircularProgress
                      color="inherit"
                      size={20}
                    />
                  ) : null}
                  {params.InputProps.endAdornment}
                </React.Fragment>
              )
            }}
          />
        )}
        disabled={isDisabled}
        autoSelect
      />
    );
  }

  let disabledButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledButtonTitle =
      "There is an ongoing operation, you can not modify the IOC right now";
  }

  return (
    <>
      <ConfirmationDialog
        title="Modifying IOC"
        description={
          <>
            <Typography component="span">
              Are you sure want to modify
              <Typography
                component="span"
                fontFamily="monospace"
              >
                {ioc.namingName}
              </Typography>{" "}
              ?
            </Typography>
          </>
        }
        confirmText="Modify IOC"
        cancelText="Cancel"
        open={open}
        onClose={onClose}
        onConfirm={onConfirm}
      />
      <SimpleAccordion
        summary="Modify IOC"
        defaultExpanded
      >
        <form
          onSubmit={onSubmit}
          style={{
            width: "100%",
            display: "flex",
            flexDirection: "column",
            gap: "0.5rem"
          }}
        >
          <AccessControl
            allowedRoles={["DeploymentToolAdmin"]}
            renderNoAccess={() => nameAutocomplete(true)}
          >
            {nameAutocomplete(iocIsDeployed)}
          </AccessControl>

          <AccessControl
            allowedRoles={["DeploymentToolAdmin"]}
            allowedUsersWithRoles={[
              { user: ioc.createdBy, role: "DeploymentToolIntegrator" }
            ]}
            renderNoAccess={() => gitRepoAutocomplete(true)}
          >
            {gitRepoAutocomplete(false)}
          </AccessControl>

          {error ? <Alert severity="error">{error}</Alert> : <></>}

          <br />
          <br />
          <Tooltip title={disabledButtonTitle}>
            <span>
              <Button
                color="primary"
                variant="contained"
                disabled={
                  requiredDataMissing() ||
                  noModification() ||
                  buttonDisabled ||
                  ioc.operationInProgress
                }
                type="submit"
              >
                Modify IOC
              </Button>
            </span>
          </Tooltip>
        </form>
      </SimpleAccordion>
    </>
  );
}
